/**
 * enb_fields.js
 * This file is part of the YATE Project http://YATE.null.ro
 *
 * Yet Another Telephony Engine - a fully featured software PBX and IVR
 * Copyright (C) 2016 Null Team
 *
 * This software is distributed under multiple licenses;
 * see the COPYING file in the main directory for licensing
 * information for this specific distribution.
 *
 * This use of this software may be subject to additional restrictions.
 * See the LEGAL file in the main directory for details.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#require "generic_validations.js"

/*---------------  EnbFields validations  ---------------*/
EnbConfig.prototype.validations =  {
    "general": {
//	"mode=": {"select":["ENB"]},
	"autostart=": {"callback": checkOnOff},
	"transceiver=": {"callback": checkOnOff}
    },
    "basic": {
	"enodebId=": {"callback": isNumber},
	"MCC=": {"regex": "^[0-9]{3}$"},
	"MNC=": {"regex": "^[0-9]{2,3}$"},
	"TAC=": {"regex": "[0-9a-zA-Z]{4}"},
	"CellIdentity=": {"callback": validateCellIdentity},
	"Name=": {"regex": "^[a-zA-Z0-9 -_]+$"},
	"Bandwidth=": {"select": ["6", "15", "25", "50", "75", "100"]},
	"NID1=": {"minimum": 0, "maximum": 167},
	"NID2=": {"minimum": 0, "maximum": 2},
	"Band=": {"select": ["1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26", "27", "28", "29", "30", "31", "32", "33", "34", "35", "36", "37", "38", "39", "40", "41", "42", "43", "44"]},
	"CrestFactor=": {"minimum": 5, "maximum": 20},
	"CFI=": {"minimum": 1, "maximum": 3},
	"Ng=": {"select":["oneSixth", "half", "one", "two"]},
	"PdcchFormat=": {"minimum": 0, "maximum": 3},
	"DistributedVrbs=": {"callback": checkOnOff},
	"SiWindowLength=": {"select": ["1", "2", "5", "10", "15", "20", "40"]},
	"SiPeriodicity=": {"select": ["8", "16", "32", "64", "128", "256", "512"]},
	"SiRedundancy=": {"minimum": 1, "maximum": 8},
	"defaultPagingCycle=": {"selected":["32","64","126","256"]},
	"nB=": {"select":["0", "32", "64", "128", "256", "512", "1024", "2048", "4096"]},
	"TxGain2=": {"minimum":0, "maximum":30},
	"RxGain1=": {"minimum":5, "maximum":30},
 	"RxGain2=": {"minimum":0, "maximum":30},
	"numberOfRA-Preambles=": {"minimum": 4, "maximum": 64},
	"preambleInitialReceivedTargetPower=": {"select":["-90","-92", "-94", "-96", "-98","-100", "-102", "-104", "-106", "-108", "-110", "-112", "-114", "-116", "-118", "-120"]},
	"rootSequenceIndex=": {"minimum": 0, "maximum": 837},
	"prach-ConfigIndex=": {"minimum": 0, "maximum": 15},
	"PrachThreshold-dB=": {"minimum": 15, "maximum": 30},
	"zeroCorrelationZoneConfig=": {"minimum": 0, "maximum": 15},
	"prach-FreqOffset=": {"minimum": 0, "maximum": 94},
	"referenceSignalPower=": {"minimum": -60, "maximum": 50},
	"groupAssignmentPUSCH=": {"minimum":0, "maximum":29},
	"cyclicShift=": {"minimum": 0, "maximum": 7},
	"p0_NominalPUSCH=": {"minimum": -126, "maximum": 24},
	"p0_NominalPUCCH=": {"minimum": -127, "maximum": -96},
	"deltaPUCCH-Shift=": {"minimum": 1, "maximum": 3},
	"nRB-CQI=": {"minimum": 0, "maximum": 98},
	"nCS-An=": {"minimum": 0, "maximum": 7},
	"srPeriodicity=": {"select":["5", "10", "20", "40", "80", "2", "1"]},
	"activity_timeout=": {"callback":checkValidInteger},
	"T302=": {"callback":checkValidInteger},
	"n1Pucch-An=": {"callback":check_n1PucchAn}
    },
    "scheduler": {
	"SpecialRntiCodeEfficiency=" : {"callback": checkSpecialRntiCodingEfficiency},
	"SpecialDCI=": {"select": ["dci1a","dci1c"]},
	"DistributedVrbs=": {"callback": checkOnOff},
	"retxBSR-Timer=": {"select":["sf320", "sf640", "sf1280", "sf2560", "sf5120", "sf10240"]},
	"periodicBSR-Timer=": {"select":["sf5", "sf10", "sf16", "sf20", "sf32", "sf40", "sf64", "sf80", "sf128", "sf160", "sf320", "sf640", "sf1280", "sf2560", "infinity"]},
	"GapType=": {"minimum": 1, "maximum": 2}
    },
    // TBI: Any other parameters in this section will be passed to the radio device
    // See the radio device's module documentation for details
    "radio": {
    },

    "uu-simulator": {
	"address=": {"callback": checkValidIP},
	"port=": {"callback": checkValidInteger},
	"pci=": {"callback": checkValidInteger},
	"earfcn=": {"callback": checkValidInteger},
	"broadcast=": {"callback": checkBroadcast},
//TBI:	"max_pdu=": {}
    },
    "uu-loopback": {
	"AddrSubnet=": {"regex": "^([0-9]{1,3}\.){3}%u$"},
	"AuxRoutingTable=": {"callback": checkAuxRoutingTable},
	"SymmetricMacCapture=": {"callback": checkOnOff}
    },
    "test-enb": {
	"PdcchTestMode=": {"callback": checkOnOff},
	"DownlinkTrafficGenererator=": {"callback": checkOnOff},
	"DownlinkTrafficGeneratorLoad=": {"minimum": 0, "maximum": 100},
	"DownlinkTrafficGeneratorSubframes=": {"minimum": 0, "maximum": 10},
// 	"SimulatePdschTraffic=": {"select": ["0", "1023", "31", "301", "602"]}, 
	"PhichEnabled=": {"callback": checkOnOff},
        "PcfichEnabled=": {"callback": checkOnOff},
	"PssEnabled=": {"callback": checkOnOff},
        "SssEnabled=": {"callback": checkOnOff},
        "CsrsEnabled=": {"callback": checkOnOff},
	"PdschEnabled=": { "callback": checkOnOff},
	"PdcchEnabled=": {"callback": checkOnOff},
	"PbchEnabled=": {"callback": checkOnOff},
	"RsEnabled=": {"callback": checkOnOff},
	"scheduler-generator=": {"callback": checkOnOff},
	"scheduler-generator.ues=": {"minimum": 1, "maximum": 10},
	"scheduler-generator.qci=": {"callback": checkValidInteger},
	"scheduler-generator.cqi=": {"callback": checkValidInteger},
	"scheduler-generator.gbr=": {"callback": checkValidInteger},
	"scheduler-generator.ambr=": {"callback": checkValidInteger},
	"scheduler-generator.upload-rate=": {"callback": checkValidInteger},
	"scheduler-generator.download-rate=": {"callback": checkValidInteger}
    },
};

function validateCellIdentity(error,field_name,field_value,section_name)
{
    if (field_value.length != 7){
	error.reason = "Field '" + field_name + "' is not a valid: " + field_value + " in section '" + section_name + "'.";
	error.error = 401;
	return false;
    }
    return true;
}

/**
 * Validate Broadcast format IP:port, IP:port, ....
 */ 
function checkBroadcast(error,field_name,field_value,section_name)
{
    var expl = field_value.split(",");
    if (!expl.length) {
	error.reason = "Invalid format for '" + field_name + "' in section '" + section_name + "'.";
	error.error = 401;
	return false;
    }
    for (var bc in expl) {
	var val = bc.split(":");
	var ip = val[0];
	var port = parseInt(val[1]);
	if (!checkValidIP(error,field_name,ip,section_name))
	    return false;

	if (port != undefined) {
	    error.reason = "Field '" + field_name + "' doesn't contain a port in section '" + section_name + "'.";
	    error.error = 401;
	    return false;
	}

	if (isNaN(port)) {
	    error.reason = "Field '" + field_name + "' doesn't contain a valid port:  '"+ port + "' in section '" + section_name + "'.";
	    error.error = 401;
	    return false;
	}
    }

    return true;
}

/* validate AuxRoutingTable field */
function  checkAuxRoutingTable(error,field_name,field_value,section_name)
{
    if (isNaN(parseInt(field_value))) { 
        if (!checkFieldValidity(error,section_name,field_name,field_value,undefined,undefined,"^[a-zA-Z0-9_-]+)$"))
	    return false;
    } else {
	if (!checkFieldValidity(error,section_name,field_name,field_value,0,255))
	    return false;
    }
    return true;
}

/* validate SpecialRntiCodingEfficiency to be float between 0.0625 - 4.0 */
function checkSpecialRntiCodingEfficiency(error,field_name,field_value,section_name)
{
    if (!checkPositiveFloat(error,field_name,field_value,section_name))
	return false;

    var min = 625;
    var max = 40000;

    var fields = field_value.split('.');

    var intPart = fields[0];
    var fracPart = fields[1];

    if (!fracPart)
        fracPart = 0;

    var fieldValue = parseInt(intPart,10) * 10000 +  parseInt(fracPart,10);
    if (fracPart.length>4)
	fieldValue += 1;

    if (fieldValue < min || fieldValue > max) {
	error.reason = "Field '" + field_name + "' is not valid: '" + field_value + "'. It has to be greater than 0.0625 and smalled than 4.0 in section '" + section_name + "'.";
	error.error = 401;
	return false;
    }
    return true;
}

/* Valid range is 0 .. (2047 * number of UL resource blocks / 110)
   Where number of UL resource blocks = Bandwitdh*/
function check_n1PucchAn(error,field_name,field_value,section_name,params)
{
    field_value = parseInt(field_value);
    var bandwidth = parseInt(params.basic["Bandwidth"]);
    var max = 2047 * bandwidth / 110;

    if (!checkFieldValidity(error,section_name,field_name,field_value,0,max))
	return false;
    return true;
}
/* vi: set ts=8 sw=4 sts=4 noet: */
